// Updates: 2003.06.13


import ecash.kernel.*;
import ecash.internet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;


public class JCash extends JFrame implements ChangeListener, ECashBuyerListener, ECashSellerListener {
  static JCash me;
  ECashBuyer eCashBuyer;
  ECashSeller eCashSeller;
  JTabbedPane tabbedPane;
  PursePane pursePane;
  PayPane payPane;
  ReceivePane receivePane;
  BankPane bankPane;
  IdPane idPane;
  HelpPane helpPane;
  AboutPane aboutPane;
  
  
  public JCash() {
    super("   JCash");
    setIconImage(new ImageIcon("System/JCashIcon16.png").getImage());      
    me = this;
    Props.load("./System/JCash");
    eCashBuyer = new ECashBuyer(this);
    eCashSeller = new ECashSeller(this);    
    JPanel p = new JPanel();
    p.setLayout(new BorderLayout());
    tabbedPane = new JTabbedPane();
    tabbedPane.addChangeListener(this);
    pursePane = new PursePane();
    tabbedPane.addTab("Purse", pursePane);
    payPane = new PayPane(eCashBuyer);
    tabbedPane.addTab("Pay", payPane);
    receivePane = new ReceivePane(eCashSeller);
    tabbedPane.addTab("Receive", receivePane);    
    bankPane = new BankPane(eCashBuyer, eCashSeller);
    tabbedPane.addTab("Bank", bankPane); 
    idPane = new IdPane();
    tabbedPane.addTab("ID", idPane); 
    helpPane = new HelpPane();
    tabbedPane.addTab("Help", helpPane); 
    aboutPane = new AboutPane();
    tabbedPane.addTab("About", aboutPane);         
    p.add(tabbedPane);
    getContentPane().add(p);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
    setSize(500, 500);
    setLocationRelativeTo(null);
    try {
      UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    }
    catch (Exception e) {System.out.println(e);}
    SwingUtilities.updateComponentTreeUI(this);
    restoreState();
    Purse.setBuyNotes(eCashBuyer.getAmountList());
    Purse.setSellNotes(eCashSeller.getAmountList());
    pursePane.setNotes();
    if (!eCashBuyer.hasBankKey()) getPublicKey();
    show();
  }


  public static void main(String[] args) {
    Splash s = new Splash(1000);
    new JCash();
    s.dispose(2000);
  }


  public static JCash getInstance() {
    return me;
  }

  
  public void stateChanged(ChangeEvent e) {
    if (receivePane != null) receivePane.closeConnection();
    if (tabbedPane.getSelectedComponent().equals(payPane)) payPane.init();
    if (tabbedPane.getSelectedComponent().equals(pursePane)) pursePane.setNotes();
  }


  public void report(PayReport payReport) {
    receivePane.report(payReport);
  }


  public void buyerPurseChanged() {
    saveState();
  }
  
  
  public void sellerPurseChanged() {
    saveState();
  }
  
  
  private void getPublicKey() {
    String bankAddress = JOptionPane.showInputDialog(null, "Enter the address of the bank\nin order to get its public key:", "   Connection to the bank", JOptionPane.QUESTION_MESSAGE);
    if (bankAddress == null) return;
    Props.set("bankAddress", bankAddress);
    WaitScreen ws = new WaitScreen(this);
    ws.setVisible(true);
    int result = eCashBuyer.getKey(bankAddress);
    ws.setVisible(false);
    if (result == ECashGen.OK) {
      eCashSeller.setKey(eCashBuyer.getKey());
      String message = "Got the public key of the bank (" + bankAddress + ").";
      JOptionPane.showMessageDialog(this, message, "OK", JOptionPane.INFORMATION_MESSAGE);  
    }
    if (result == ECashGen.SERVER_NOT_READY) {
      String message = "Server of the bank is not available !";      
      JOptionPane.showMessageDialog(this, message, "Error", JOptionPane.ERROR_MESSAGE);
    }
    if (result == ECashGen.KEY_NOT_AVAILABLE) {
      String message = "Public key of the bank not available yet !";
      JOptionPane.showMessageDialog(this, message, "Error", JOptionPane.ERROR_MESSAGE);
    }
    saveState();
  }    
  
  
  private void saveState() {
    Vector v = new Vector();
    v.add(eCashBuyer.getState());
    v.add(eCashSeller.getState());
    Tools.writeObject("./System/JCashState.obj", v);
  }      


  private void restoreState() {
    Vector v = (Vector)Tools.readObject("./System/JCashState.obj");
    if (v == null) return;
    eCashBuyer.setState((ECashBuyerState)v.get(0));
    eCashSeller.setState((ECashSellerState)v.get(1));
  }
}