// Updates: 2003.06.13


import ecash.kernel.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;


public class MoneyInputField extends JTextField implements KeyListener {
  Vector listeners;
  String sourceName;
  String regexp;
  
  
  public MoneyInputField(int size, String regexp) {
    super(size);
    this.regexp = regexp;
    addKeyListener(this);
    listeners = new Vector();
  }


  public MoneyInputField(int size, String regexp, String sourceName) {
    this(size, regexp);
    this.sourceName = sourceName;
    setText(Props.get(sourceName));
  }


  public void addMoneyInputFieldListener(MoneyInputFieldListener listener) {
    listeners.add(listener);
  }
 
 
  public Vector getList() {
    Vector res = new Vector();
    String[] entry = getText().replaceAll(" ", "").split("-");
    for (int i = 0; i < entry.length; i++) {
      int sep = entry[i].indexOf("(");
      if (sep == -1) res.add("" + Double.parseDouble(entry[i]));
      else {
        String noteValue = entry[i].substring(0, sep);
        int mult = Integer.parseInt(entry[i].substring(sep + 1, entry[i].indexOf(")")));
        for (int j = 0; j < mult; j++) {
          res.add("" + Double.parseDouble(noteValue));
        }
      }
    }
    return res;
  }
  
  
  public boolean isValid() {
    String[] entry = getText().replaceAll(" ", "").split("-");
    for (int i = 0; i < entry.length; i++) {
      if (!entry[i].matches(regexp)) return false;
    }
    return true;
  }


  public void removeMoneyInputFieldListener(MoneyInputFieldListener listener) {  
    listeners.remove(listener);
  } 

    
  private void fireNewTotal(EventObject e, String text) {
    for (int i = 0; i < listeners.size(); i++) {
      ((MoneyInputFieldListener)listeners.elementAt(i)).newTotal(e, text);
    }
  }
  

  private double treatEntry(String anEntry) {  
    int sep = anEntry.indexOf("(");
    if (sep == -1) return Double.parseDouble(anEntry);
    double note = Double.parseDouble(anEntry.substring(0, sep));
    double mult = Double.parseDouble(anEntry.substring(sep + 1, anEntry.indexOf(")")));
    return note * mult;
  }
  
    
  public void keyReleased(KeyEvent e) {
    boolean ok = true;
    double sum = 0.0;
    String[] entry = getText().replaceAll(" ", "").split("-");
    for (int i = 0; i < entry.length; i++) {
      if (entry[i].matches(regexp)) sum += treatEntry(entry[i]);
      else {
        ok = false;
        break;
      }
    }
    if (sourceName != null) Props.set(sourceName, getText());
    if (ok) fireNewTotal(e, "" + Tools.round(sum)); 
    else fireNewTotal(e, null);
  }
  
  
  public void keyPressed(KeyEvent e) {}    
  public void keyTyped(KeyEvent e) {}     
}  