// Updates: 2003.06.13


package ecash.internet;


import ecash.kernel.*;


/**
 * Such a report is returned to a client when he buys or deposits some
 * money at the bank. 
 * @author Michel Deriaz
 * @see ECashBuyer#buy
 * @see ECashSeller#deposit
 */
public class CreditDebitReport {
  private String currentAmount = null;
  private int result = -1; 
  

  /**
   * Creates a report.
   */   
  public CreditDebitReport() {
  }
  

  /**
   * Returns the current amount of money available in the bank account.
   * @return the current amount of money available in the bank account
   */
  public String getCurrentAmount() {
    return "" + Tools.round(currentAmount);
  }


  /**
   * Returns an int that represents how the operation succeeded.
   *
   * <p>For a buy operation, it can be 
   * <br>ECashGen.OK,
   * <br>ECashGen.NOTES_EXP_SYNTAX_ERROR,
   * <br>ECashGen.SERVER_NOT_READY,
   * <br>ECashGen.CHEAT_WRONG_SIGNATURE or
   * <br>ECashGen.CONNECTION_CLOSED_BY_SERVER.
   *
   * <p>For a deposit operation, it can be
   * <br>ECashGen.OK, 
   * <br>ECashGen.NOTES_EXP_SYNTAX_ERROR,
   * <br>ECashGen.NOTES_NOT_AVAILABLE,
   * <br>ECashGen.SERVER_NOT_READY or
   * <br>ECashGen.CONNECTION_CLOSED_BY_SERVER.
   *
   * @return an int that represents how the operation succeeded
   * @see ECashGen
   */ 
  public int getResult() {
    return result;
  }


  /**
   * Sets the current amount available on the bank account.
   * @param currentAmount the current amount
   */ 
  public void setCurrentAmount(String currentAmount) {
    this.currentAmount = currentAmount;
  }    
  

  /**
   * Set the result that describes how the operation succeeded.
   * @param result an integer describing how the operation succeeded
   * @see #getResult
   */ 
  public void setResult(int result) {
    this.result = result;
  }  
}  