// Updates: 2003.06.13


package ecash.internet;


/**
 * This interface must be implemented by every class that creates an 
 * ECashBank object.
 * @author Michel Deriaz
 * @see ECashBank
 */
public interface ECashBankListener {
  

  /**
   * Returns true if the password correspond with the account number, false
   * otherwise.
   * @param accountNumber the account number
   * @param password the password
   * @return a boolean indicating whether the password corresponds to the 
   * account number or not
   */   
  public boolean checkPassword(String accountNumber, String password);
  
  
  /**
   * Returns the available amount in the specified bank account.
   * @param accountNumber the account to check
   * @return the available amount
   */   
  public String getAmount(String accountNumber);
  

  /**
   * Called after a client deposited some money on his bank account.
   * @param depositReport a report describing the operation
   * @see DepositReport
   */   
  public void report(DepositReport depositReport);
  

  /**
   * Called after a client bought some money from his bank account.
   * @param buyReport a report describing the operation
   * @see BuyReport
   */     
  public void report(BuyReport buyReport);
}