// Updates: 2003.06.13


package ecash.internet;


import ecash.kernel.*;
import java.io.*;


/**
 * Represents the state of an ECashBank object. It contains it's key ring
 * (private and public keys) and the list of all the former desposited
 * notes, in order to avoid that people use several times the same note.
 * Such an object is meant to be stored on a hard-disk as a backup.
 * @author Michel Deriaz
 * @see ECashBank
 */
public class ECashBankState implements Serializable {
  private KeyRing keyRing;
  private UsedNotesList usedNotesList;
  

  /**
   * Constructs an ECashBankState.
   * @param keyRing the key ring
   * @param usedNotesList an object containing all the former desposited
   * notes 
   */   
  public ECashBankState(KeyRing keyRing, UsedNotesList usedNotesList) {
    this.keyRing = keyRing;
    this.usedNotesList = usedNotesList;
  }
  

  /**
   * Gets the key ring.
   * @return the key ring   
   */   
  public KeyRing getKeyRing() {
    return keyRing;
  }
  
  /**
   * Gets the object containing all the former desposited notes.
   * @return the object containing all the former desposited notes
   */  
  public UsedNotesList getUsedNotesList() {
    return usedNotesList;
  }
}