// Updates: 2003.06.13


package ecash.internet;


import ecash.kernel.*;
import java.io.*;


/**
 * Represents the state of an ECashBuyer object. It contains the public key
 * of the bank and a BuyerPurse object. Such an object is meant to be 
 * stored on a hard-disk as a backup.
 * @author Michel Deriaz
 * @see ECashBuyer
 */
public class ECashBuyerState implements Serializable {
  private BuyerPurse buyerPurse;
  private PubKey pubKey;
  
  
  /**
   * Constructs an ECashBuyerState.
   * @param buyerPurse the buyer purse
   * @param pubKey the public key of the bank
   */    
  public ECashBuyerState(BuyerPurse buyerPurse, PubKey pubKey) {
    this.buyerPurse = buyerPurse;
    this.pubKey = pubKey;
  }
  

  /**
   * Returns the buyer purse.
   * @return the buyer purse
   */   
  public BuyerPurse getBuyerPurse() {
    return buyerPurse;
  }
  

  /**
   * Returns the public key of the bank.
   * @return the public key of the bank
   */   
  public PubKey getPubKey() {
    return pubKey;
  }
}