// Updates: 2003.06.13


package ecash.internet;


import ecash.kernel.*;
import java.io.*;


/**
 * Represents the state of an ECashSeller object. It contains the public
 * key of the bank and a SellerPurse object. Such an object is meant to be 
 * stored on a hard-disk as a backup.
 * @author Michel Deriaz
 * @see ECashSeller
 */
public class ECashSellerState implements Serializable {
  private SellerPurse sellerPurse;
  private PubKey pubKey;
  

  /**
   * Constructs an ECashSellerState.
   * @param sellerPurse the seller purse
   * @param pubKey the public key of the bank
   */    
  public ECashSellerState(SellerPurse sellerPurse, PubKey pubKey) {
    this.sellerPurse = sellerPurse;
    this.pubKey = pubKey;
  }
  

  /**
   * Returns the seller purse.
   * @return the seller purse
   */     
  public SellerPurse getSellerPurse() {
    return sellerPurse;
  }
  

  /**
   * Returns the public key of the bank.
   * @return the public key of the bank
   */     
  public PubKey getPubKey() {
    return pubKey;
  }
}