// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;
import java.io.*;


/**
 * Represents a number chosen by the bank, with n = [0..nbEnveloppe[. The 
 * client will have to reveal all information used to build the notes,
 * except for the note n (which will be blindly signed and returned).
 * @author Michel Deriaz
 */
public class BankChoice implements Serializable {
  private BigInteger choice;

  
  /**
   * Chooses randomly a number which represents the choice of the bank.
   * @param nbEnveloppes the number of envelopes 
   */
  public BankChoice(int nbEnvelopes) {
    choice = new BigInteger("" + (int)(nbEnvelopes*Math.random()));    
  }


  /**
   * Re-build the object from it's string representation.
   * @param value the string representation of the object
   */
  public BankChoice(String value) {
    choice = new BigInteger(value);
  }
  
  
  /**
   * Re-build the object from it's file representation.
   * @param filename the filename
   */
  public static BankChoice load(String filename) {
    return new BankChoice(Tools.readFile(filename));
  }
  

  /**
   * Saves this object into it's file representation.
   * @param filename the filename
   */
  public void save(String filename) {
    Tools.writeFile(filename, toString());
  } 
  

  /**
   * Returns an int representation of this BankChoice.
   * @return an int representation of this BankChoice
   */  
  public int toInt() {
    return choice.intValue();
  }
  
  
  /**
   * Returns a string representation of this BankChoice.
   * @return a string representation of this BankChoice
   */
  public String toString() {
    return choice.toString();
  }
}