// Updates: 2003.06.13


package ecash.kernel;


import java.io.*;
import java.util.*;


/**
 * This class loads some parameters from a properties file, and make them
 * available through corresponding static fields.
 * @author Michel Deriaz
 */
public class Const {
  
  
  /**
   * Directory used to store the file representation of some objects. The
   * value is used by the readFile() and writeFile() methods in the Tools
   * class.
   * @see Tools#readFile
   * @see Tools#writeFile
   */
  public static String DIR;
  
  
  /**
   * The size of the key (modulus) used by the bank to sign notes. Because
   * the keys are generated by the RSA algorithm, this value must be
   * within [512..2048].
   */  
  public static int KEY_SIZE;
  
  
  /**
   * The number of envelopes that are sent to the bank for each note.
   */  
  public static int NB_ENVELOPE;
  
  
  /**
   * The number of RIS pairs.
   */   
  public static int NB_RIS;
  
  
  /**
   * The regular expression that defines a notes expression. Can also be 
   * used by a program in order to control the user's inputs. 
   */  
  public static String REGEXP;
  
  
  private Const() {
  }
  
  
  /**
   * Loads the values from the specified filename.
   * @param filenamePart the filename without extension
   */  
  public static void load(String filenamePart) {
    String filename = "ecash/kernel/" + filenamePart + ".properties";
    Properties properties = new Properties();
    try {
      properties.load(ClassLoader.getSystemResourceAsStream(filename));
    }
    catch (IOException e) {
      System.out.println("Error in Props.load(): " + e);
    }
    DIR = properties.getProperty("DIR");
    KEY_SIZE = Integer.parseInt(properties.getProperty("KEY_SIZE"));
    NB_ENVELOPE = Integer.parseInt(properties.getProperty("NB_ENVELOPE"));
    NB_RIS = Integer.parseInt(properties.getProperty("NB_RIS"));
    REGEXP = properties.getProperty("REGEXP");
  }
}