// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;
import java.io.*;


/**
 * This class represents a single note with all it's related information:
 * envelope, blinding factor, RIS, ...
 * @author Michel Deriaz
 */  
public class FullNote implements Serializable {
  private BigInteger bigM;
  private BigInteger m;
  private BigInteger envelope;
  private BigInteger amount;
  private BigInteger k;
  private BigInteger[] x;
  private BigInteger[] y;
  

  /**
   * Creates a FullNote.
   * @param bigM the note
   * @param m a signed hash of the note
   * @param envelope the envelope used to hide the note during the blind
   * signature process
   * @param amount the amount of the note
   * @param k the blinding factor
   * @param x the RIS
   * @param y the hash of the RIS
   */   
  public FullNote(BigInteger bigM, BigInteger m, BigInteger envelope, BigInteger amount, BigInteger k, BigInteger[] x, BigInteger[] y) {
    this.bigM = bigM;
    this.m = m;
    this.envelope = envelope;
    this.amount = amount;
    this.k = k;
    this.x = x;
    this.y = y;
  }
  

  /**
   * Returns bigM, the note.
   * @return bigM
   */   
  public BigInteger getBigM() {
    return bigM;
  }
  

  /**
   * Returns m, the signed hash of the note.
   * @return m
   */    
  public BigInteger getM() {
    return m;
  }
  

  /**
   * Returns envelope, the envelope used to hide the note during the blind
   * signature process
   * @return envelope
   */  
  public BigInteger getEnvelope() {
    return envelope;
  }
  

  /**
   * Returns amount, the amount of the note.
   * @return amount
   */  
  public BigInteger getAmount() {
    return amount;
  }
  

  /**
   * Returns k, the blinding factor.
   * @return k
   */  
  public BigInteger getK() {
    return k;
  }
  

  /**
   * Returns x, the RIS.
   * @return x
   */    
  public BigInteger[] getX() {
    return x;
  }
  

  /**
   * Returns y, the hash of the RIS.
   * @return y
   */    
  public BigInteger[] getY() {
    return y;
  }
}