// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;
import java.io.*;


/**
 * Public key of the bank, used to sign the bank notes.
 * @author Michel Deriaz
 */
public class PubKey implements Serializable {
  private BigInteger e;
  private BigInteger n;
  
  
  /**
   * Creates a public key from the public exponent and the modulus.
   * @param e the public exponent
   * @param n the modulus
   */   
  public PubKey(BigInteger e, BigInteger n) {
    this.e = e;
    this.n = n;
  }


  /**
   * Re-build the object from it's string representation.
   * @param value the string representation of the object
   */
  public PubKey(String value) {
    String[] tockens = value.split("\n");
    e = new BigInteger(tockens[0]);
    n = new BigInteger(tockens[1]);
  }
  

  /**
   * Returns the public exponent of the key.
   * @return e, the public exponent of the key
   */ 
  public BigInteger getE() {
    return e;
  }


  /**
   * Returns the modulus of the key.
   * @return n, the modulus of the key
   */ 
  public BigInteger getN() {
    return n;
  }
  

  /**
   * Re-build the object from it's file representation.
   * @param filename the filename
   */  
  public static PubKey load(String filename) {
    return new PubKey(Tools.readFile(filename));
  }
  

  /**
   * Saves this object into it's file representation.
   * @param filename the filename
   */  
  public void save(String filename) {
    Tools.writeFile(filename, toString());
  }  
   

  /**
   * Returns a string representation of this PubKey.
   * @return a string representation of this PubKey
   */      
  public String toString() {
    return "" + e + "\n" + n;
  }
}