// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;
import java.io.*;


/**
 * The revealed part of the RIS that the buyer send to the seller in order
 * to prove the validity of the note.
 * @author Michel Deriaz
 */
public class RISPart implements Serializable {
  private BigInteger[] xRevealed;
  

  /**
   * Creates a RISPart from the information found in the FullNotesList
   * object. The bank choice designate which note has been chosen and
   * signed by the bank.
   * @param fullNotesList the full notes list
   * @param bankChoice the choice of the bank
   * @param challengeRIS the challenge
   */ 
  public RISPart(FullNotesList fullNotesList, BankChoice bankChoice, BigInteger challengeRIS) {
    BigInteger[] x = fullNotesList.get(bankChoice.toInt()).getX();
    xRevealed = new BigInteger[x.length/2];
    for (int i = 0; i < xRevealed.length; i++) {
      if (challengeRIS.testBit(i)) xRevealed[i] = x[i*2+1];
      else xRevealed[i] = x[i*2];
    }
  }
  

  /**
   * Re-build the object from it's string representation.
   * @param value the string representation of the object
   */  
  public RISPart(String value) {
    String[] tockens = value.split(" ");
    xRevealed = new BigInteger[tockens.length];
    for (int i = 0; i < xRevealed.length; i++) {
      xRevealed[i] = new BigInteger(tockens[i]);
    }
  }


  /**
   * Returns the revealed part of the RIS.
   * @return the revealed part of the RIS
   */   
  public BigInteger[] getValues() {
    return xRevealed;
  }
  
  
  
  /**
   * Re-build the object from it's file representation.
   * @param filename the filename
   */   
  public static RISPart load(String filename) {
    return new RISPart(Tools.readFile(filename));
  }
    
  
  /**
   * Saves this object into it's file representation.
   * @param filename the filename
   */  
  public void save(String filename) {
    Tools.writeFile(filename, toString());
  }
    
  
  /**
   * Returns a string representation of this RISPart.
   * @return a string representation of this RISPart
   */  
  public String toString() {
    String result = "";
    for (int i = 0; i < xRevealed.length; i++) {
      result += xRevealed[i] + " ";
    }
    return result;
  }  
}