// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;


/**
 * This class is used by buyers to reveal to the bank the secret
 * information used to build a note. Buyers will have to send such an
 * object to the bank for every note except for the one choosen by the bank
 * (which will be signed).
 * @author Michel Deriaz
 * @see RevealedNotesList
 */
public class RevealedNote {
  private BigInteger amount;
  private BigInteger k;
  private BigInteger[] x;
  

  /**
   * Creates a RevealedNote.
   * @param amount the amount of the note
   * @param k the blinding factor
   * @param x the RIS
   */ 
  public RevealedNote(BigInteger amount, BigInteger k, BigInteger[] x) {
    this.amount = amount;
    this.k = k;
    this.x = x;
  }
  

  /**
   * Creates a RevealedNote object from the information found in a FullNote
   * object.
   * @param fullNote the full note
   */     
  public RevealedNote(FullNote fullNote) {
    amount = fullNote.getAmount();
    k = fullNote.getK();
    x = fullNote.getX();
  }
 
  
  /**
   * Returns the amount of the note.
   * @return amount, the amount of the note
   */ 
  public BigInteger getAmount() {
    return amount;
  }
  

  /**
   * Returns the blinding factor.
   * @return k, the blinding factor
   */ 
  public BigInteger getK() {
    return k;
  }
  
  
  /**
   * Returns the RIS.
   * @return x, the RIS
   */   
  public BigInteger[] getX() {
    return x;
  }
}