// Updates: 2003.06.13


package ecash.kernel;


import java.io.*;
import java.math.*;


/**
 * This class represents a single item of the SellerPurse class.
 * @author Michel Deriaz
 * @see SellerPurse
 */
public class SellerPurseUnit implements Serializable {
  private Note note;
  private BigInteger challengeRIS;
  private RISPart risPart;
  private String buyerNickname;
  

  /**
   * Creates a SellerPurseUnit.
   * @param note the note
   * @param challengeRIS the challenge
   * @param risPart the revealed part of the RIS
   * @param buyerNickname the nickname provided by the buyer in order to 
   * identify the origin of the payment
   */    
  public SellerPurseUnit(Note note, BigInteger challengeRIS, RISPart risPart, String buyerNickname) {
    this.note = note;
    this.challengeRIS = challengeRIS;
    this.risPart = risPart;
    this.buyerNickname = buyerNickname;
  }
  
  
  /**
   * Returns the note.
   * @return the note
   */    
  public Note getNote() {
    return note;
  }
  
  
  /**
   * Returns the challenge.
   * @return the challenge
   */   
  public BigInteger getChallengeRIS() {
    return challengeRIS;
  }
  
  
  /**
   * Returns the revealed part of the RIS.
   * @return the revealed part of the RIS
   */   
  public RISPart getRISPart() {
    return risPart;
  }
  
  
  /**
   * Returns the nickname of the buyer.
   * @return the nickname of the buyer
   */   
  public String getBuyerNickname() {
    return buyerNickname;
  }
}