// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;
import java.io.*;


/**
 * This class represents a single item of the UsedNotesList class. 
 * @author Michel Deriaz
 * @see UsedNotesList
 */
public class UsedNote implements Serializable {
  private BigInteger bigM;
  private BigInteger[] xRevealed;
  
  
  /**
   * Creates a UsedNote. In addition to the note (bigM), the RIS that the
   * seller got during the transaction with the buyer is also recorded;
   * this will be useful to identify a buyer if the later tries to cheat by
   * re-using several times the same note.
   * @param bigM the note
   * @param xRevealed the revealed RIS that the seller got during the
   * transaction with the buyer
   */   
  public UsedNote(BigInteger bigM, BigInteger[] xRevealed) {
    this.bigM = bigM;
    this.xRevealed = xRevealed;
  }

  
  /**
   * Returns the note.
   * @return bigM, the note
   */   
  public BigInteger getBigM() {
    return bigM;
  }
  

  /**
   * Returns the revealed part of the RIS.
   * @return xRevealed, the revealed part of the RIS
   */   
  public BigInteger[] getX() {
    return xRevealed;
  }
}