// Updates: 2003.06.13


import ecash.kernel.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.math.*;
import java.util.*;


public class Alice extends JFrame implements ActionListener {
  public static final String ID = "555";

  private JButton makeNotes, revealList, unblind, revealRIS;
  private JTextField tfAmount, tfPubKey, tfEnvelopesList;
  private JTextField tfBankChoice, tfRevealedNotesList;
  private JTextField tfSignedEnvelope, tfNote;
  private JTextField tfChallengeRIS, tfRISPart;
  
  private PubKey pubKey;
  private FullNotesList fullNotesList;
  private BankChoice bankChoice;
  
  
  public Alice(boolean centered) {
    super("   Alice");

    JPanel p = new JPanel(new GridLayout(0, 3, 4, 4));
    p.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    
    makeNotes = new JButton("Make notes");
    makeNotes.addActionListener(this);
    p.add(makeNotes);
    tfAmount = new JTextField("20");
    tfAmount.setToolTipText("in: amount");
    p.add(tfAmount);   
    tfPubKey = new JTextField("PubKey.txt");
    tfPubKey.setToolTipText("in: public key of the bank");
    p.add(tfPubKey);
    p.add(new Container());                
    tfEnvelopesList = new JTextField("EnvelopesList.txt");
    tfEnvelopesList.setToolTipText("out: BigIntegers that represents the envelopes to sign");
    p.add(tfEnvelopesList);
    p.add(new Container());

    revealList = new JButton("Reveal list");
    revealList.addActionListener(this);
    p.add(revealList);
    tfBankChoice = new JTextField("BankChoice.txt");
    tfBankChoice.setToolTipText("in: all the other envelopes have to be completely revealed");
    p.add(tfBankChoice);
    tfRevealedNotesList = new JTextField("RevealedNotesList.txt");
    tfRevealedNotesList.setToolTipText("out: informations used to create the envelopes (without the choosen one)");
    p.add(tfRevealedNotesList);
    
    unblind = new JButton("Unblind");
    unblind.addActionListener(this);
    p.add(unblind);
    tfSignedEnvelope = new JTextField("SignedEnvelope.txt");
    tfSignedEnvelope.setToolTipText("in: this signed (by the bank) envelope contains the wanted note");
    p.add(tfSignedEnvelope);
    tfNote = new JTextField("Note.txt");
    tfNote.setToolTipText("out: the signed note, ready to use");
    p.add(tfNote);

    revealRIS = new JButton("Reveal RIS");
    revealRIS.addActionListener(this);
    p.add(revealRIS);
    tfChallengeRIS = new JTextField("ChallengeRIS.txt");
    tfChallengeRIS.setToolTipText("in: the challenge, as a succession of bits");
    p.add(tfChallengeRIS);
    tfRISPart = new JTextField("RISPart.txt");
    tfRISPart.setToolTipText("out: the asked part of the RIS");
    p.add(tfRISPart);
      
    getContentPane().add(p);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
    setSize(415, 200);
    if (centered) setLocationRelativeTo(null);
    else setLocation(700, 335);
    try {
      UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    }
    catch (Exception e) {System.out.println(e);}
    SwingUtilities.updateComponentTreeUI(this);
    
    Const.load("ECash");
    restoreState();
    
    show();
  }


  public static void main(String[] args) {
    new Alice(true);
  }
  
  
  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(makeNotes)) makeNotes();
    if (e.getSource().equals(revealList)) revealList();
    if (e.getSource().equals(unblind)) unblind();
    if (e.getSource().equals(revealRIS)) revealRIS();
    saveState();
  }  


  private void makeNotes() {
    String amount = tfAmount.getText();
    pubKey = PubKey.load(tfPubKey.getText());    
    fullNotesList = new FullNotesList(Const.NB_ENVELOPE, amount, ID, pubKey);
    EnvelopesList envelopesList = fullNotesList.getEnvelopesList();
    envelopesList.save(tfEnvelopesList.getText());
  }
  

  private void revealList() {
    bankChoice = BankChoice.load(tfBankChoice.getText());
    RevealedNotesList revealedNotesList = new RevealedNotesList(fullNotesList, bankChoice);
    revealedNotesList.save(tfRevealedNotesList.getText());
  }
 
  
  private void unblind() {
    BigInteger signedEnvelope = new BigInteger(Tools.readFile(tfSignedEnvelope.getText()));
    BigInteger bigM = fullNotesList.get(bankChoice.toInt()).getBigM();
    BigInteger signedNote = Tools.unblind(signedEnvelope, fullNotesList, bankChoice, pubKey);
    Note note = new Note(bigM, signedNote);
    if (!note.checkSignature(pubKey)) {
      JOptionPane.showMessageDialog(this, "The signature is not correct !", "Error", JOptionPane.ERROR_MESSAGE);
    }
    note.save(tfNote.getText());
  }
  
   
  private void revealRIS() {
    BigInteger challengeRIS = new BigInteger(Tools.readFile(tfChallengeRIS.getText()));
    RISPart risPart = new RISPart(fullNotesList, bankChoice, challengeRIS);
    risPart.save(tfRISPart.getText());
  }
  

  private void restoreState() {
    Vector v = (Vector)Tools.readObject("AliceState.obj");
    if (v == null) return;
    pubKey = (PubKey)v.get(0);
    fullNotesList = (FullNotesList)v.get(1);
    bankChoice = (BankChoice)v.get(2);
  }

  
  private void saveState() {
    Vector v = new Vector();
    v.add(pubKey);
    v.add(fullNotesList);
    v.add(bankChoice);
    Tools.writeObject("AliceState.obj", v);
  }  
}