// Updates: 2003.06.13


import ecash.kernel.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.math.*;
import java.util.*;



public class Bank extends JFrame implements ActionListener {
  private JButton genKey, choose, sign, checkNote;
  private JTextField tfPubKey;
  private JTextField tfEnvelopesList, tfBankChoice;
  private JTextField tfRevealedNotesList, tfSignedEnvelope;
  private JTextField tfNote, tfChallengeRIS, tfRISPart, tfUsedNotesList;
  
  private KeyRing keyRing;
  private PubKey pubKey;
  private BankChoice bankChoice;
  private EnvelopesList envelopesList;
  
  
  public Bank(boolean centered) {
    super("   Bank");

    JPanel p = new JPanel(new GridLayout(0, 3, 4, 4));
    p.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    
    genKey = new JButton("Gen. Keys");
    genKey.addActionListener(this);
    p.add(genKey);
    tfPubKey = new JTextField("PubKey.txt");
    tfPubKey.setToolTipText("out: public key of the bank");
    p.add(tfPubKey);   
    p.add(new Container());   
    
    choose = new JButton("Choose");
    choose.addActionListener(this);
    p.add(choose);
    tfEnvelopesList = new JTextField("EnvelopesList.txt");
    tfEnvelopesList.setToolTipText("in: BigIntegers that represents the envelopes to sign");
    p.add(tfEnvelopesList);
    tfBankChoice = new JTextField("BankChoice.txt");
    tfBankChoice.setToolTipText("out: all the other envelopes have to be completely revealed");
    p.add(tfBankChoice);
    
    sign = new JButton("Sign");
    sign.addActionListener(this);
    p.add(sign);
    tfRevealedNotesList = new JTextField("RevealedNotesList.txt");
    tfRevealedNotesList.setToolTipText("in: informations used to create the envelopes (without the choosen one)");
    p.add(tfRevealedNotesList);
    tfSignedEnvelope = new JTextField("SignedEnvelope.txt");
    tfSignedEnvelope.setToolTipText("out: this signed (by the bank) envelope contains the wanted note");
    p.add(tfSignedEnvelope);

    checkNote = new JButton("Check note");
    checkNote.addActionListener(this);
    p.add(checkNote);
    tfNote = new JTextField("Note.txt");
    tfNote.setToolTipText("in: the signed note");
    p.add(tfNote);
    tfChallengeRIS = new JTextField("ChallengeRIS.txt");
    tfChallengeRIS.setToolTipText("in: the challenge, as a succession of bits");
    p.add(tfChallengeRIS);
    p.add(new Container());   
    tfRISPart = new JTextField("RISPart.txt");
    tfRISPart.setToolTipText("in: the asked part of the RIS");
    p.add(tfRISPart);
    tfUsedNotesList = new JTextField("UsedNotesList.txt");
    tfUsedNotesList.setToolTipText("in / out: list of all the used notes");
    p.add(tfUsedNotesList);
    
    getContentPane().add(p);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
    setSize(415, 200);
    if (centered) setLocationRelativeTo(null);
    else setLocation(700, 120);
    try {
      UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    }
    catch (Exception e) {System.out.println(e);}
    SwingUtilities.updateComponentTreeUI(this);
    
    Const.load("ECash");
    restoreState();
    
    show();
  }


  public static void main(String[] args) {
    new Bank(true);
  }
  
  
  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(genKey)) genKey();
    if (e.getSource().equals(choose)) choose();
    if (e.getSource().equals(sign)) sign();    
    if (e.getSource().equals(checkNote)) checkNote();    
    saveState();
  }  


  private void genKey() {
    keyRing = new KeyRing();
    pubKey = keyRing.getPublic();
    pubKey.save(tfPubKey.getText());
  }
 
 
  private void choose() {
    envelopesList = EnvelopesList.load(tfEnvelopesList.getText());
    bankChoice = new BankChoice(Const.NB_ENVELOPE);
    bankChoice.save(tfBankChoice.getText());
  }
  
  
  private void sign() {
    RevealedNotesList revealedNotesList = RevealedNotesList.load(tfRevealedNotesList.getText());
    String amount = revealedNotesList.getAmount();
    if (amount.equals("-1")) {
      JOptionPane.showMessageDialog(this, "The amounts are not all the same !", "Error", JOptionPane.ERROR_MESSAGE);    
    }    
    if (!envelopesList.checkRevealed(revealedNotesList, bankChoice, Alice.ID, pubKey)) {
      JOptionPane.showMessageDialog(this, "The revealed notes are not correct !", "Error", JOptionPane.ERROR_MESSAGE);    
    }
    BigInteger signedEnvelope = Tools.signEnvelope(envelopesList.get(bankChoice.toInt()), keyRing);
    Tools.writeFile(tfSignedEnvelope.getText(), signedEnvelope.toString());
    System.out.println("Bank: Signed a note of " + amount);
  }    
    
    
  private void checkNote() {
    Note note = Note.load(tfNote.getText());
    if (!note.checkSignature(pubKey)) {
      JOptionPane.showMessageDialog(this, "The signature is not correct !", "Error", JOptionPane.ERROR_MESSAGE);
    }
    BigInteger challengeRIS = new BigInteger(Tools.readFile(tfChallengeRIS.getText()));                
    RISPart risPart = RISPart.load(tfRISPart.getText());
    if (!note.checkRIS(challengeRIS, risPart)) {
      JOptionPane.showMessageDialog(this, "The RIS is not correct !", "Error", JOptionPane.ERROR_MESSAGE);
    }
    UsedNotesList usedNotesList = UsedNotesList.load(tfUsedNotesList.getText());
    int answer = usedNotesList.check(note.getBigM(), risPart.getValues());
    if (answer == -1) {
      JOptionPane.showMessageDialog(this, "The seller already used this note !", "Error", JOptionPane.ERROR_MESSAGE);
    }
    else if (answer > 0) {
      JOptionPane.showMessageDialog(this, "The buyer with account n " + answer + " already used this note !", "Error", JOptionPane.ERROR_MESSAGE);
    }  
    else {      
      usedNotesList.add(note.getBigM(), risPart.getValues());
      usedNotesList.save(tfUsedNotesList.getText());
    }
    System.out.println("Bank: Got a note of " + note.getAmount());
  }

  
  private void restoreState() {
    Vector v = (Vector)Tools.readObject("BankState.obj");
    if (v == null) return;
    keyRing = (KeyRing)v.get(0);
    pubKey = (PubKey)v.get(1);
    bankChoice = (BankChoice)v.get(2);
    envelopesList = (EnvelopesList)v.get(3);
  }

  
  private void saveState() {
    Vector v = new Vector();
    v.add(keyRing);
    v.add(pubKey);
    v.add(bankChoice);
    v.add(envelopesList);
    Tools.writeObject("BankState.obj", v);
  }
}