// Updates: 2003.06.13


import ecash.kernel.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.math.*;
import java.util.*;
import java.security.*;


public class Bob extends JFrame implements ActionListener {
  private JButton challenge, checkRIS;
  private JTextField tfPubKey, tfNote, tfChallengeRIS;
  private JTextField tfRISPart;
    
  private Note note;
  private BigInteger challengeRIS;
    
  
  public Bob(boolean centered) {
    super("   Bob");

    JPanel p = new JPanel(new GridLayout(0, 3, 4, 4));
    p.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    
    challenge = new JButton("Challenge");
    challenge.addActionListener(this);
    p.add(challenge);
    tfPubKey = new JTextField("PubKey.txt");
    tfPubKey.setToolTipText("in: public key of the bank");
    p.add(tfPubKey);
    tfNote = new JTextField("Note.txt");
    tfNote.setToolTipText("in: the note");
    p.add(tfNote);   
    p.add(new Container());    
    tfChallengeRIS = new JTextField("ChallengeRIS.txt");
    tfChallengeRIS.setToolTipText("out: the challenge, as a succession of bits");
    p.add(tfChallengeRIS);
    p.add(new Container());     

    checkRIS = new JButton("Check RIS");
    checkRIS.addActionListener(this);
    p.add(checkRIS);
    tfRISPart = new JTextField("RISPart.txt");
    tfRISPart.setToolTipText("in: the asked values from the RIS");
    p.add(tfRISPart);
    
    getContentPane().add(p);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
    setSize(415, 140);
    if (centered) setLocationRelativeTo(null);
    else setLocation(700, 550);
    try {
      UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    }
    catch (Exception e) {System.out.println(e);}
    SwingUtilities.updateComponentTreeUI(this);

    Const.load("ECash");
    restoreState();
        
    show();
  }


  public static void main(String[] args) {
    new Bob(true);
  }
  
  
  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(challenge)) challenge();
    if (e.getSource().equals(checkRIS)) checkRIS();
    saveState();    
  }  


  private void challenge() {
    PubKey pubKey = PubKey.load(tfPubKey.getText());
    note = Note.load(tfNote.getText());
    if (!note.checkSignature(pubKey)) {
      JOptionPane.showMessageDialog(this, "The signature is not correct !", "Error", JOptionPane.ERROR_MESSAGE);
    }
    System.out.println("Bob: Got a note of " + note.getAmount());        
    challengeRIS = new BigInteger(Const.NB_RIS, new SecureRandom());
    Tools.writeFile(tfChallengeRIS.getText(), challengeRIS.toString());
  }
  
  
  private void checkRIS() {
    RISPart risPart = RISPart.load(tfRISPart.getText());
    if (!note.checkRIS(challengeRIS, risPart)) {
      JOptionPane.showMessageDialog(this, "The RIS is not correct !", "Error", JOptionPane.ERROR_MESSAGE);
    }
  }

    
  private void restoreState() {
    Vector v = (Vector)Tools.readObject("BobState.obj");
    if (v == null) return;
    note = (Note)v.get(0);
    challengeRIS = (BigInteger)v.get(1);
  }

  
  private void saveState() {
    Vector v = new Vector();
    v.add(note);
    v.add(challengeRIS);
    Tools.writeObject("BobState.obj", v);
  }    
}