// Updates: 2003.06.13


import ecash.kernel.*;
import ecash.internet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;


public class Bank extends JFrame implements ActionListener, ECashBankListener {
  private ECashBank eCashBank;
  private TextZone tz;
  private JButton genKey, createAccount, showAccounts;
  private ArrayList list;
   

  public Bank(boolean centered) {
    super("   Bank");
    JPanel p = new JPanel();
    p.setLayout(new BorderLayout());
    tz = new TextZone();
    p.add(tz, BorderLayout.CENTER);
    JPanel buttonsPanel = new JPanel();
    genKey = new JButton("Gen. keys");
    genKey.addActionListener(this);
    buttonsPanel.add(genKey);
    createAccount = new JButton("Create account");
    createAccount.addActionListener(this);
    buttonsPanel.add(createAccount);
    showAccounts = new JButton("Show accounts");
    showAccounts.addActionListener(this);
    buttonsPanel.add(showAccounts);    
    p.add(buttonsPanel, BorderLayout.SOUTH);
    getContentPane().add(p);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
    setSize(370, 300);
    if (centered) setLocationRelativeTo(null);
    else setLocation(390, 320);
    try {
      UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    }
    catch (Exception e) {System.out.println(e);}
    SwingUtilities.updateComponentTreeUI(this);
        
    list = new ArrayList();
    eCashBank = new ECashBank(this);
    restoreState();
    eCashBank.startServer();

    show();
  }
  

  public static void main(String[] args) {
    new Bank(true);
  }
  
  
  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(genKey)) genKey();
    if (e.getSource().equals(createAccount)) createAccount();
    if (e.getSource().equals(showAccounts)) showAccounts();
    saveState();
  }
  
  
  private void genKey() {
    eCashBank.genKey();
    tz.append("New pair of key generated.\n");
  }
  
  
  private void createAccount() {
    CreateAccountDialog dialog = new CreateAccountDialog(this);
    if (!dialog.isCanceled()) {
      String number = dialog.getNumber();
      String password = dialog.getPassword();
      list.add(new Account(number, password));
      tz.append("New account created: (" + number + ", " + password + ")\n");
    }
  }
  
  
  public void showAccounts() {
    tz.append("List of all the accounts:\n");
    for (int i = 0; i < list.size(); i++) {
      Account account = (Account)list.get(i);
      tz.append("   (" + account.getNumber() + ", " + account.getPassword() + ")\n");
    }
  }


  public boolean checkPassword(String accountNumber, String password) {
    for (int i = 0; i < list.size(); i++) {
      Account account = (Account)list.get(i);
      if (account.getNumber().equals(accountNumber) && account.getPassword().equals(password)) return true;
    }
    return false;
  }
  
  
  public String getAmount(String accountNumber) {
    return "0.0"; 
      // this program don't manage the amounts, but this method must be 
      // implemented because of the ECashBankListener interface
  }
  
  
  public void report(BuyReport buyReport) {
    String accountNumber = buyReport.getAccountNumber();
    String sum = buyReport.computeSum();
    String notesExp = buyReport.computeNotesExp();
    int result = buyReport.getResult();
    if (result == ECashGen.OK) tz.append("N " + accountNumber + " bought " + sum + " {" + notesExp + "}\n");
    if (result == ECashGen.CHEAT_WRONG_PASSWORD) tz.append("N " + accountNumber + " cheats: Wrong account number or password !\n");
    if (result == ECashGen.CHEAT_AMOUNTS_NOT_ALL_THE_SAME) tz.append("N " + accountNumber + " cheats: The amounts of at least one note are not all the same !\n");
    if (result == ECashGen.CHEAT_WRONG_REVEALED_NOTES) tz.append("N " + accountNumber + " cheats: The revealed notes are not correct !\n");
    saveState();
  }
      
  
  public void report(DepositReport depositReport) {
    String accountNumber = depositReport.getAccountNumber();
    String cheaterAccountNumber = depositReport.getCheaterAccountNumber();
    String sum = depositReport.computeSum();
    String notesExp = depositReport.computeNotesExp();
    int result = depositReport.getResult();
    if (result == ECashGen.OK) tz.append("N " + accountNumber + " deposited " + sum + " {" + notesExp + "}\n");
    if (result == ECashGen.CHEAT_WRONG_PASSWORD) tz.append("N " + accountNumber + " cheats: Wrong account number or password !\n");
    if (result == ECashGen.CHEAT_WRONG_SIGNATURE) tz.append("N " + accountNumber + " cheats: Wrong signature !\n");
    if (result == ECashGen.CHEAT_WRONG_RIS) tz.append("N " + accountNumber + " cheats: Wrong RIS !\n");
    if (result == ECashGen.CHEAT_NOTE_REUSED_BY_SELLER) tz.append("N " + accountNumber + " cheats: Note reused by seller !\n");
    if (result == ECashGen.CHEAT_NOTE_REUSED_BY_BUYER) tz.append("N " + accountNumber + " got a note already used by N " + cheaterAccountNumber + " !\n");
    saveState();
  }
  
  
  private void restoreState() {
    Vector v = (Vector)Tools.readObject("BankState.obj");
    if (v == null) return;
    eCashBank.setState((ECashBankState)v.get(0));
    list = (ArrayList)v.get(1);
  }

  
  private void saveState() {
    Vector v = new Vector();
    v.add(eCashBank.getState());
    v.add(list);
    Tools.writeObject("BankState.obj", v);
  }  
}