// Updates: 2003.06.13


import ecash.kernel.*;
import ecash.internet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;


public class Buyer extends JFrame implements ActionListener, ECashBuyerListener {
  private ECashBuyer eCashBuyer;  
  private TextZone tz;
  private JButton getKey, notesList, buy, pay;
   

  public Buyer(boolean centered) {
    super("   Buyer");
    JPanel p = new JPanel();
    p.setLayout(new BorderLayout());
    tz = new TextZone();
    p.add(tz, BorderLayout.CENTER);
    JPanel buttonsPanel = new JPanel();
    getKey = new JButton("Get key");
    getKey.addActionListener(this);
    buttonsPanel.add(getKey);    
    notesList = new JButton("Notes list");
    notesList.addActionListener(this);
    buttonsPanel.add(notesList);        
    buy = new JButton("Buy");
    buy.addActionListener(this);
    buttonsPanel.add(buy);
    pay = new JButton("Pay");
    pay.addActionListener(this);
    buttonsPanel.add(pay);
    p.add(buttonsPanel, BorderLayout.SOUTH);
    getContentPane().add(p);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
    setSize(370, 300);
    if (centered) setLocationRelativeTo(null);
    else setLocation(770, 130);
    try {
      UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    }
    catch (Exception e) {System.out.println(e);}
    SwingUtilities.updateComponentTreeUI(this);
    
    eCashBuyer = new ECashBuyer(this);
    restoreState();
    
    show();
  }


  public static void main(String[] args) {
    new Buyer(true);
  }
  
  
  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(getKey)) getKey();
    if (e.getSource().equals(notesList)) notesList();
    if (e.getSource().equals(buy)) buy();
    if (e.getSource().equals(pay)) pay();
  }
  
  
  public void buyerPurseChanged() {
    saveState();
  }
  
  
  private void getKey() {
    GetKeyDialog dialog = new GetKeyDialog(this, "Buyer");
    if (!dialog.isCanceled()) {
      String bankAddress = dialog.getBankAddress();
      int result = eCashBuyer.getKey(bankAddress);
      if (result == ECashGen.OK) tz.append("Got the public key of the bank (" + bankAddress + ").\n");
      if (result == ECashGen.SERVER_NOT_READY) tz.append("Server of the bank is not available !\n");      
      if (result == ECashGen.KEY_NOT_AVAILABLE) tz.append("Public key of the bank not available yet !\n");      
    }    
    saveState();
 }
  
  
  private void buy() {
    if (!eCashBuyer.hasBankKey()) {
      JOptionPane.showMessageDialog(this, "You need first to get the public key of the bank.", "Error", JOptionPane.ERROR_MESSAGE);    
      return;
    }
    BuyDialog dialog = new BuyDialog(this, "Buyer");
    if (!dialog.isCanceled()) {
      String bankAddress = dialog.getBankAddress();
      String bankAccount = dialog.getBankAccount();
      char[] password = dialog.getPassword();
      String notesExp = dialog.getNotesExp();
      CreditDebitReport report = eCashBuyer.buy(bankAddress, bankAccount, password, notesExp);
      int result = report.getResult();
      if (result == ECashGen.OK) tz.append("Bought {" + notesExp + "} from " + bankAddress + ", account N " + bankAccount + "\n");
      if (result == ECashGen.NOTES_EXP_SYNTAX_ERROR) tz.append("Syntax error in the notes list !\n"); 
      if (result == ECashGen.SERVER_NOT_READY) tz.append("Server of the bank is not available !\n");      
      if (result == ECashGen.CHEAT_WRONG_SIGNATURE) tz.append("The current note isn't correctly signed !\n");
      if (result == ECashGen.CONNECTION_CLOSED_BY_SERVER) tz.append("Connection closed by server.\n");
    }
  }
  
  
  private void pay() {
    PayDialog dialog = new PayDialog(this, "Buyer");
    if (!dialog.isCanceled()) {
      String nickname = dialog.getNickname();
      String sellerAddress = dialog.getSellerAddress();
      String notesExp = dialog.getNotesExp();
      int result = eCashBuyer.pay(sellerAddress, nickname, notesExp);
      if (result == ECashGen.OK) tz.append("Sold {" + notesExp + "} to " + sellerAddress + "\n");
      if (result == ECashGen.NOTES_EXP_SYNTAX_ERROR) tz.append("Syntax error in the notes list !\n"); 
      if (result == ECashGen.NOTES_NOT_AVAILABLE) tz.append("The requested notes are not all available !\n");      
      if (result == ECashGen.SERVER_NOT_READY) tz.append("Server of the seller is not available !\n");      
      if (result == ECashGen.CONNECTION_CLOSED_BY_SERVER) tz.append("Connection closed by server.\n");
    }
  }


  private void notesList() {
    Vector v = eCashBuyer.getAmountList();
    if (v.size() < 1) {
      tz.append("No notes !\n");
      return;
    }
    tz.append("List of all the notes:\n");
    for (int i = 0; i < v.size(); i++) {
      tz.append("   " + (String)v.get(i) + "\n");
    }
  }
 
 
  private void restoreState() {
    Vector v = (Vector)Tools.readObject("BuyerState.obj");
    if (v == null) return;
    eCashBuyer.setState((ECashBuyerState)v.get(0));
  }

  
  private void saveState() {
    Vector v = new Vector();
    v.add(eCashBuyer.getState());
    Tools.writeObject("BuyerState.obj", v);
  }    
}