// Updates: 2003.06.13


import ecash.kernel.*;
import ecash.internet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;


public class Seller extends JFrame implements ActionListener, ECashSellerListener {
  private ECashSeller eCashSeller;  
  private TextZone tz;
  private JButton getKey, notesList, deposit, startServer, stopServer;
   

  public Seller(boolean centered) {
    super("   Seller");
    JPanel p = new JPanel();
    p.setLayout(new BorderLayout());
    tz = new TextZone();  
    p.add(tz, BorderLayout.CENTER);
    JPanel buttonsPanel = new JPanel(new GridLayout(2, 1));
    JPanel buttonsPanel1 = new JPanel();
    JPanel buttonsPanel2 = new JPanel();
    getKey = new JButton("Get key");
    getKey.addActionListener(this);
    buttonsPanel1.add(getKey);       
    notesList = new JButton("Notes list");
    notesList.addActionListener(this);
    buttonsPanel1.add(notesList);         
    deposit = new JButton("Deposit");
    deposit.addActionListener(this);
    buttonsPanel1.add(deposit);
    startServer = new JButton("Start server");
    startServer.addActionListener(this);
    buttonsPanel2.add(startServer);
    stopServer = new JButton("Stop server");
    stopServer.addActionListener(this);
    buttonsPanel2.add(stopServer);        
    buttonsPanel.add(buttonsPanel1);
    buttonsPanel.add(buttonsPanel2);
    p.add(buttonsPanel, BorderLayout.SOUTH);
    getContentPane().add(p);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
    setSize(370, 300);
    if (centered) setLocationRelativeTo(null);
    else setLocation(770, 440);
    try {
      UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    }
    catch (Exception e) {System.out.println(e);}
    SwingUtilities.updateComponentTreeUI(this);
        
    eCashSeller = new ECashSeller(this);
    restoreState();
    
    show();
  }


  public static void main(String[] args) {
    new Seller(true);
  }
  
  
  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(getKey)) getKey();
    if (e.getSource().equals(notesList)) notesList();
    if (e.getSource().equals(deposit)) deposit();
    if (e.getSource().equals(startServer)) startServer();
    if (e.getSource().equals(stopServer)) stopServer();
  }
  
  
  private void getKey() {
    GetKeyDialog dialog = new GetKeyDialog(this, "Seller");
    if (!dialog.isCanceled()) {
      String bankAddress = dialog.getBankAddress();
      int result = eCashSeller.getKey(bankAddress);
      if (result == ECashGen.OK) tz.append("Got the public key of the bank (" + bankAddress + ").\n");
      if (result == ECashGen.SERVER_NOT_READY) tz.append("Server of the bank is not available !\n");      
      if (result == ECashGen.KEY_NOT_AVAILABLE) tz.append("Public key of the bank not available yet !\n");      
    }    
    saveState();
  }


  private void deposit() {
    DepositDialog dialog = new DepositDialog(this, "Seller");
    if (!dialog.isCanceled()) {
      String bankAddress = dialog.getBankAddress();
      String bankAccount = dialog.getBankAccount();
      char[] password = dialog.getPassword();
      String notesExp = dialog.getNotesExp();
      CreditDebitReport report = eCashSeller.deposit(bankAddress, bankAccount, password, notesExp);
      int result = report.getResult();      
      if (result == ECashGen.OK) tz.append("Deposited {" + notesExp + "} at " + bankAddress + ", account N " + bankAccount + "\n");
      if (result == ECashGen.NOTES_EXP_SYNTAX_ERROR) tz.append("Syntax error in the notes list !\n"); 
      if (result == ECashGen.NOTES_NOT_AVAILABLE) tz.append("The requested notes are not all available !\n");
      if (result == ECashGen.SERVER_NOT_READY) tz.append("Server of the bank is not available !\n");      
      if (result == ECashGen.CONNECTION_CLOSED_BY_SERVER) tz.append("Connection closed by server.\n");      
    }
  }
  
    
  private void startServer() {
    if (!eCashSeller.hasBankKey()) {
      JOptionPane.showMessageDialog(this, "You need first to get the public key of the bank.", "Error", JOptionPane.ERROR_MESSAGE);    
      return;
    }    
    eCashSeller.startServer();
    tz.append("Server started.\n");
  }
  

  private void stopServer() {
    eCashSeller.stopServer();
    tz.append("Server stopped.\n");
  }  
  

  private void notesList() {
    Vector v = eCashSeller.getAmountList();
    if (v.size() < 1) {
      tz.append("No notes !\n");
      return;
    }
    tz.append("List of all the notes:\n");
    for (int i = 0; i < v.size(); i++) {
      tz.append("   " + (String)v.get(i) + "\n");
    }
  }


  public void sellerPurseChanged() {
    saveState();
  }
  
  
  public void report(PayReport payReport) {
    String buyerNickname = payReport.getBuyerNickname();
    String sum = payReport.computeSum();
    String notesExp = payReport.computeNotesExp();    
    int result = payReport.getResult();    
    if (result == ECashGen.OK) tz.append(buyerNickname + " has given " + sum + " {" + notesExp + "}\n");
    if (result == ECashGen.CHEAT_WRONG_SIGNATURE) tz.append(buyerNickname + " cheats: Wrong signature !\n");
    if (result == ECashGen.CHEAT_WRONG_RIS) tz.append(buyerNickname + " cheats: Wrong RIS !\n");
    tz.append("Server stopped.\n");
  }  
  
  
  private void restoreState() {
    Vector v = (Vector)Tools.readObject("SellerState.obj");
    if (v == null) return;
    eCashSeller.setState((ECashSellerState)v.get(0));
  }

  
  private void saveState() {
    Vector v = new Vector();
    v.add(eCashSeller.getState());
    Tools.writeObject("SellerState.obj", v);
  }      
}