// Updates: 2003.06.13


import ecash.kernel.*;
import ecash.internet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;


public class JBank extends JFrame implements ActionListener, MouseListener, MenuListener, ECashBankListener {
  public static final int SORT_BY_NUMBER = 0;
  public static final int SORT_BY_USERNAME = 1;
  public static final int SORT_BY_AMOUNT = 2;
  public static int sortKey = SORT_BY_NUMBER;  
  
  private ECashBank eCashBank;
  private AccountsList accountsList;
  private TextZone tz;
  private Vector list;
  private JMenuItem miGenKey, miCreditAccount, miNewAccount, miModifyAccount, miDeleteAccount, miQuit;
  private JMenuItem miTips, miAbout;
  private JRadioButtonMenuItem numberItem, usernameItem, amountItem;
  private JPopupMenu popup;
  private JMenuItem miCreditAccount2, miNewAccount2, miModifyAccount2, miDeleteAccount2;
     

  public JBank(boolean centered) {
    super("   JBank");
    setIconImage(new ImageIcon("System/JBankIcon16.png").getImage());  
    JPanel p = new JPanel();
    p.setLayout(new GridLayout(2, 1));
    accountsList = new AccountsList();
    accountsList.addMouseListener(this);
    p.add(accountsList);
    tz = new TextZone();    
    p.add(tz);
    getContentPane().add(p);
    
    JMenuBar menuBar = new JMenuBar();
    setJMenuBar(menuBar); 

    JMenu sortMenu = new JMenu("Sort");
    sortMenu.setMnemonic('S');
    ButtonGroup group = new ButtonGroup();
    numberItem = new JRadioButtonMenuItem("by account number");
    numberItem.setSelected(true);
    numberItem.addActionListener(this);
    group.add(numberItem);
    sortMenu.add(numberItem);
    usernameItem = new JRadioButtonMenuItem("by username");
    usernameItem.addActionListener(this);
    group.add(usernameItem);    
    sortMenu.add(usernameItem);
    amountItem = new JRadioButtonMenuItem("by amount");
    amountItem.addActionListener(this);
    group.add(amountItem);    
    sortMenu.add(amountItem);    
    
    JMenu fileMenu = new JMenu("File");
    fileMenu.addMenuListener(this);
    fileMenu.setMnemonic('F');
    miGenKey = fileMenu.add("Generate keys");
    miGenKey.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.CTRL_MASK));
    miGenKey.setMnemonic('G');
    miGenKey.addActionListener(this);
    miCreditAccount = fileMenu.add("Credit account...");
    miCreditAccount.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_K, InputEvent.CTRL_MASK));
    miCreditAccount.setMnemonic('C');
    miCreditAccount.addActionListener(this);
    miNewAccount = fileMenu.add("New account...");
    miNewAccount.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
    miNewAccount.setMnemonic('N');
    miNewAccount.addActionListener(this);
    miModifyAccount = fileMenu.add("Modify account...");
    miModifyAccount.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_MASK));
    miModifyAccount.setMnemonic('M');
    miModifyAccount.addActionListener(this);    
    miDeleteAccount = fileMenu.add("Delete account");
    miDeleteAccount.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_MASK));
    miDeleteAccount.setMnemonic('D');
    miDeleteAccount.addActionListener(this);        
    fileMenu.add(sortMenu);
    fileMenu.addSeparator();
    miQuit = fileMenu.add("Quit");
    miQuit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_MASK));
    miQuit.setMnemonic('Q');
    miQuit.addActionListener(this);        
    menuBar.add(fileMenu);
    
    JMenu helpMenu = new JMenu("Help");
    helpMenu.setMnemonic('H');
    miTips = helpMenu.add("Tips");
    miTips.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
    miTips.setMnemonic('T');
    miTips.addActionListener(this);
    miAbout = helpMenu.add("About");
    miAbout.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0));
    miAbout.setMnemonic('A');
    miAbout.addActionListener(this);
    menuBar.add(helpMenu);
        
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    
    setSize(370, 500);
    if (centered) setLocationRelativeTo(null);
    else setLocation(390, 320);
    try {
      UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    }
    catch (Exception e) {System.out.println(e);}
    SwingUtilities.updateComponentTreeUI(this);
    
    list = new Vector();
    eCashBank = new ECashBank(this);
    restoreState();
    accountsList.setValues(list);
    eCashBank.startServer();
    buildPopup();
    sort();
    
    show();
  }
  

  public static void main(String[] args) {
    Splash s = new Splash(1000);
    new JBank(true);
    s.dispose(2000);
  }


  private void buildPopup() {
    popup = new JPopupMenu();
    miCreditAccount2 = popup.add("Credit account...");
    miCreditAccount2.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_K, InputEvent.CTRL_MASK));
    miCreditAccount2.setMnemonic('C');
    miCreditAccount2.addActionListener(this);
    miNewAccount2 = popup.add("New account...");
    miNewAccount2.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
    miNewAccount2.setMnemonic('N');
    miNewAccount2.addActionListener(this);
    miModifyAccount2 = popup.add("Modify account...");
    miModifyAccount2.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_MASK));
    miModifyAccount2.setMnemonic('M');
    miModifyAccount2.addActionListener(this);    
    miDeleteAccount2 = popup.add("Delete account");
    miDeleteAccount2.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_MASK));
    miDeleteAccount2.setMnemonic('D');
    miDeleteAccount2.addActionListener(this);            
  }
    
  
  public void actionPerformed(ActionEvent e) {
    Object s = e.getSource();
    if (s.equals(miGenKey)) genKey();
    if (s.equals(miCreditAccount) || s.equals(miCreditAccount2)) creditAccount();
    if (s.equals(miNewAccount)    || s.equals(miNewAccount2))    newAccount();
    if (s.equals(miModifyAccount) || s.equals(miModifyAccount2)) modifyAccount();
    if (s.equals(miDeleteAccount) || s.equals(miDeleteAccount2)) deleteAccount();
    if (s.equals(numberItem)) sort(SORT_BY_NUMBER);
    if (s.equals(usernameItem)) sort(SORT_BY_USERNAME);
    if (s.equals(amountItem)) sort(SORT_BY_AMOUNT);
    if (s.equals(miQuit))   System.exit(0);
    if (s.equals(miTips))   tips();
    if (s.equals(miAbout))  about();
    sort();
    saveState();
  }
  
  
  private void genKey() {
    eCashBank.genKey();
    tz.append("New pair of key generated.\n");
  }
  
    
  private void creditAccount() {
    Account account = accountsList.getSelected();
    if (account == null) return;
    double currentAmount = account.getAmountDouble();
    String message = "Account: " + account.getAccountNumber() + "\nUsername: " + account.getUsername() + "\nAmount: " + account.getAmount()
                     + "\n\nHow much would you like to credit (or debit) ?";
    String res = JOptionPane.showInputDialog(this, message, "Credit an account", JOptionPane.QUESTION_MESSAGE);    
    if (res == null) return;
    try {
      double resDouble = Double.parseDouble(res);
      account.setAmount(currentAmount + resDouble);
      tz.append("The banker credited account N " + account.getAccountNumber() + " with " + res + "\n");
    }
    catch (NumberFormatException e2) {
      JOptionPane.showMessageDialog(this, "Invalid number format !", "Error", JOptionPane.ERROR_MESSAGE);
    }
  }
    
  
  private void newAccount() {
    int maxAccountNumber = 0;
    for (int i = 0; i < list.size(); i++) {
      int accountNumber = Integer.parseInt(((Account)list.get(i)).getAccountNumber());
      if (accountNumber > maxAccountNumber) maxAccountNumber = accountNumber;
    }  
    NewAccountDialog nad = new NewAccountDialog(this, "" + (maxAccountNumber+1));
    if (nad.isCanceled()) return;
    list.add(new Account(nad.getAccountNumber(), nad.getUsername(), nad.getPassword(), nad.getAmount()));
    accountsList.setValues(list);
    tz.append("The banker created a new account: (" + nad.getAccountNumber() + ", " +  nad.getUsername() + ", " + nad.getPassword() + ", " +  nad.getAmount() + ")\n");
  }
  
  
  private void modifyAccount() {
    Account account = accountsList.getSelected();
    if (account == null) return;        
    NewAccountDialog nad = new NewAccountDialog(this, account);
    if (nad.isCanceled()) return;
    account.setUsername(nad.getUsername());
    account.setPassword(nad.getPassword());
    account.setAmount(nad.getAmount());
    tz.append("The banker modified an account: (" + nad.getAccountNumber() + ", " +  nad.getUsername() + ", " + nad.getPassword() + ", " +  nad.getAmount() + ")\n");
  }
    

  private void deleteAccount() {
    Account account = accountsList.getSelected();
    if (account == null) return;
    String message = "Account: " + account.getAccountNumber() + "\nUsername: " + account.getUsername() + "\nAmount: " + account.getAmount()
                       + "\n\nYou really want to delete this account ?\n\n\n";
    int returnVal = JOptionPane.showConfirmDialog(this, message, "Select an Option", JOptionPane.YES_NO_OPTION);
    if (returnVal == JOptionPane.NO_OPTION) return;        
    list.remove(account);
    accountsList.setValues(list);
    tz.append("The banker deleted an account: (" + account.getAccountNumber() + ", " +  account.getUsername() + ", " + account.getPassword() + ", " +  account.getAmount() + ")\n");
  }
  
  
  private void sort(int sortKey) {
    this.sortKey = sortKey;
  }


  private void tips() {
    String msg = "";
    msg += "- Use the popup menu (right buttton on the mouse) to access quickly to the main functions.\n\n";
    msg += "- Double-click over an item to credit or debit the associated account.\n\n";
    JOptionPane.showMessageDialog(this, msg, "   Tips", JOptionPane.INFORMATION_MESSAGE);
  }  

  
  private void about() {
    new AboutDialog(this);
  }
 

  public boolean checkPassword(String accountNumber, String password) {
    for (int i = 0; i < list.size(); i++) {
      Account account = (Account)list.get(i);
      if (account.getAccountNumber().equals(accountNumber) && account.getPassword().equals(password)) return true;
    }
    return false;
  }
  
  
  public String getAmount(String accountNumber) {
    for (int i = 0; i < list.size(); i++) {
      Account account = (Account)list.get(i);
      if (account.getAccountNumber().equals(accountNumber)) return account.getAmount();
    }
    return null;    
  }
  
  
  public void report(BuyReport buyReport) {
    String accountNumber = buyReport.getAccountNumber();
    String sum = buyReport.computeSum();
    String notesExp = buyReport.computeNotesExp();
    int result = buyReport.getResult();
    if (result == ECashGen.OK) {
      creditAccount(accountNumber, "-"+sum);
      tz.append("N " + accountNumber + " bought " + sum + " {" + notesExp + "}\n");
    }
    if (result == ECashGen.CHEAT_WRONG_PASSWORD) tz.append("N " + accountNumber + " cheats: Wrong account number or password !\n");
    if (result == ECashGen.CHEAT_AMOUNTS_NOT_ALL_THE_SAME) tz.append("N " + accountNumber + " cheats: The amounts of at least one note are not all the same !\n");
    if (result == ECashGen.CHEAT_WRONG_REVEALED_NOTES) tz.append("N " + accountNumber + " cheats: The revealed notes are not correct !\n");
    sort();
    saveState();
  }
      
  
  public void report(DepositReport depositReport) {
    String accountNumber = depositReport.getAccountNumber();
    String cheaterAccountNumber = depositReport.getCheaterAccountNumber();
    String sum = depositReport.computeSum();
    String notesExp = depositReport.computeNotesExp();
    int result = depositReport.getResult();
    if (result == ECashGen.OK) {
      creditAccount(accountNumber, sum);
      tz.append("N " + accountNumber + " deposited " + sum + " {" + notesExp + "}\n");
    }
    if (result == ECashGen.CHEAT_WRONG_PASSWORD) tz.append("N " + accountNumber + " cheats: Wrong account number or password !\n");
    if (result == ECashGen.CHEAT_WRONG_SIGNATURE) tz.append("N " + accountNumber + " cheats: Wrong signature !\n");
    if (result == ECashGen.CHEAT_WRONG_RIS) tz.append("N " + accountNumber + " cheats: Wrong RIS !\n");
    if (result == ECashGen.CHEAT_NOTE_REUSED_BY_SELLER) tz.append("N " + accountNumber + " cheats: Note reused by seller !\n");
    if (result == ECashGen.CHEAT_NOTE_REUSED_BY_BUYER) tz.append("N " + accountNumber + " got a note already used by N " + cheaterAccountNumber + " !\n");
    sort();
    saveState();
  }
  
  
  public void mouseClicked(MouseEvent e) {
    if (e.getClickCount() == 2) {
      creditAccount();
      sort();
      saveState();
    }
  }
  
  
  public void mouseReleased(MouseEvent e) {
    if (e.isPopupTrigger()) {
      if (accountsList.getSelected() == null) {
        miCreditAccount2.setEnabled(false);
        miModifyAccount2.setEnabled(false);
        miDeleteAccount2.setEnabled(false);
      }
      else {
        miCreditAccount2.setEnabled(true);
        miModifyAccount2.setEnabled(true);
        miDeleteAccount2.setEnabled(true);
      }
      popup.show(e.getComponent(), e.getX(), e.getY());
    }
  }
  
  
  public void menuSelected(MenuEvent evt) {
    if (accountsList.getSelected() == null) {
      miCreditAccount.setEnabled(false);
      miModifyAccount.setEnabled(false);
      miDeleteAccount.setEnabled(false);
    }
    else {
      miCreditAccount.setEnabled(true);
      miModifyAccount.setEnabled(true);
      miDeleteAccount.setEnabled(true);
    }    
  }


  private void sort() {
    Collections.sort(list);
    accountsList.setValues(list);
  }
  
  
  private void creditAccount(String accountNumber, String sum) {
    for (int i = 0; i < list.size(); i++) {
      Account account = (Account)list.get(i);
      if (account.getAccountNumber().equals(accountNumber)) {
        double current = Double.parseDouble(account.getAmount());
        double diff = Double.parseDouble(sum);
        account.setAmount(current + diff);
        return;
      }
    }
  }
  
 
  private void restoreState() {
    Vector v = (Vector)Tools.readObject("./System/BankState.obj");
    if (v == null) return;
    eCashBank.setState((ECashBankState)v.get(0));
    list = (Vector)v.get(1);
  }

  
  private void saveState() {
    Vector v = new Vector();
    v.add(eCashBank.getState());
    v.add(list);
    Tools.writeObject("./System/BankState.obj", v);
  }  
  

  public void mousePressed(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}    
  
  public void menuDeselected(MenuEvent evt) {}
  public void menuCanceled(MenuEvent evt) {}  
}