// Updates: 2003.02.15, 2003.06.13


/* 
 * This classe shows an Index. Each time that the user entries a new 
 * string, all the listeners that implement the IndexListener interface
 * will be notified.
 */
 

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;


public class Index extends JPanel implements ActionListener, KeyListener, MouseListener  {
  private final String INDEX_FILE = "./System/Contacts.txt";
  private boolean secondLine;
  private JTextField tf;
  private JList li;
  private JButton delete;
  private Vector v;
  private JScrollPane scrollPane;
  private Vector listeners;
  private IndexMyCellRenderer cellRenderer;
  

  /*
   * Constructs an index.
   */
  public Index() {
    setLayout(new BorderLayout());
    setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    JPanel p = new JPanel();
    p.setLayout(new BorderLayout());
    p.setBorder(BorderFactory.createTitledBorder("  Contacts  "));
    tf = new JTextField();
    tf.addKeyListener(this);
    tf.addActionListener(this);    
    p.add(tf, BorderLayout.NORTH);
    li = new JList();
    li.addMouseListener(this);
    li.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    cellRenderer = new IndexMyCellRenderer();
    li.setCellRenderer(cellRenderer);    
    scrollPane = new JScrollPane(li);
    p.add(scrollPane, BorderLayout.CENTER);    
    JPanel panDel = new JPanel();
    panDel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    delete = new JButton("Delete contact");
    delete.addActionListener(this);
    panDel.add(delete);
    p.add(panDel, BorderLayout.SOUTH);
    add(p, BorderLayout.CENTER);
    fillList(INDEX_FILE);
    saveList(INDEX_FILE);
    listeners = new Vector();
  }
  

  /*
   * Adds an item.
   */
  public void add(String item) {
    v.add(item);
    Collections.sort(v);
    li.setListData(v);  
    saveList(INDEX_FILE);
    fillList(INDEX_FILE);  
    saveList(INDEX_FILE);          
  }
      

  /*
   * Adds an listener that will be notified for each entry.
   */
  public void addIndexListener(IndexListener listener) {
    listeners.add(listener);
  }
 

  /*
   * Adds the content of the specified file to the current list. If a 
   * particular item already exists, the latter is ignored.
   */
  public void addList(String filename) {
    String text = readFile(filename) + "\n";
    StringTokenizer st = new StringTokenizer(text, "\r\n");
    String item;
    while (st.hasMoreTokens()) {
      item = st.nextToken();
      if (!v.contains(item)) v.add(item);
    }
    Collections.sort(v);
    li.setListData(v);    
    saveList(INDEX_FILE);
  }  
  

  /*
   * Removes all the items of the list.
   */
  public void clear() {
    v = new Vector();
    li.setListData(v); 
    setText("");
    toBlack();
    saveList(INDEX_FILE);
  }
  
  
  /*
   * Returns true if the specified item is in the list, false else.
   */
  public boolean contains(String item) {
    return v.contains(item);
  }
  
  
  /*
   * Removes the selected item from the list.
   */  
  public void deleteSelectedItem() {
    v.remove(li.getSelectedValue());
    li.setListData(v);    
    saveList(INDEX_FILE);
  }
  
  
  /*
   * Exports the current list to the specified file.
   */
  public void exportList(String filename) {
    saveList(filename);
  }
  
  
  /*
   * Returns the selected item as a String.
   */
  public String getSelectedString() {
    return (String)li.getSelectedValue();
  }

  
  /*
   * Initializes the index.
   */
  public void init() {
    setText("");
    toBlack();
    li.clearSelection();
    JScrollBar sb = scrollPane.getVerticalScrollBar();
    sb.setValue(0);          
    EventQueue.invokeLater(new Runnable() {
      public void run() {
        requestFocus();
      }
    });
  }
  
  
  /*
   * If secondLine is true, the scrollbar will be set so that the selected
   * item will be located at the second line. Otherwise, the scrollbar will
   * just be set if necessary, to ensure that the selected item is visible.
   */
  public void putSelectedAtSecondLine(boolean secondLine) {
    this.secondLine = secondLine;
  }
  

  /*
   * Removes a listener.
   */
  public void removeIndexListener(IndexListener listener) {  
    listeners.remove(listener);
  }  
    

  /*
   * Deletes in the current list all the items from the specified file.
   */
  public void removeList(String filename) {
    String text = readFile(filename) + "\n";
    StringTokenizer st = new StringTokenizer(text, "\r\n");
    String item;
    while (st.hasMoreTokens()) {
      item = st.nextToken();
      if (v.contains(item)) v.remove(item);
    }
    Collections.sort(v);
    li.setListData(v);    
    saveList(INDEX_FILE);
  }
  

  /*
   * Sets the entry text.
   */
  public void setText(String text) {
    tf.setText(text);
  }
 
   
  /*
   * Sets the color black for the text.
   */
  public void toBlack() {
    tf.setForeground(Color.black);
    cellRenderer.toBlack();
  }
  

  /*
   * Requests the focus for the entry textfield.
   */
  public void requestFocus() {
    tf.requestFocus();   
  }
  

  public void actionPerformed(ActionEvent e) {
    if (e.getSource().equals(delete)) fireDeleteItem();
    else treatEntry();
  }


  public void keyReleased(KeyEvent e) {
    String text = tf.getText();
    if (e.getKeyCode() == KeyEvent.VK_INSERT) fireNewInput(false, text);
    else if (e.getKeyCode() == KeyEvent.VK_UP) select(false);
    else if (e.getKeyCode() == KeyEvent.VK_DOWN) select(true);
    else checkInput(text);
  }
  

  public void mouseReleased(MouseEvent e) {
    treatEntry();  
    tf.requestFocus();  
  }
  

  private void fillList(String name) {
    String text = readFile(name) + "\n";
    v = new Vector();
    StringTokenizer st = new StringTokenizer(text, "\r\n");
    String item;
    while (st.hasMoreTokens()) {
      item = st.nextToken();
      if (!v.contains(item)) v.add(item);
    }
    Collections.sort(v);
    li.setListData(v);    
  }
  
  
  private void saveList(String name) {
    String text = "";
    for (int i = 0; i < v.size(); i++) {
      text += (String)v.elementAt(i) + "\r\n";
    }
    writeFile(name, text);
  }
      
  
  private String readFile(String name) {
    File f = new File(name);
    int size = (int)f.length ();
    char cbuf[] = new char[size];
    try {
      FileReader fr = new FileReader(name);    
      fr.read(cbuf);
      fr.close();
    }
    catch(IOException ioe) {
      return null;
    }
    return String.valueOf(cbuf);
  }  
  
  
  private void treatEntry() {
    if (v.size() == 0) return;
    if (li.getSelectedIndex() == -1) fireNewInput(false, tf.getText());
    else fireNewInput(true, (String)li.getSelectedValue());
  }
  
 
  private void writeFile(String name, String text) {
    try {
      FileWriter fw = new FileWriter(name);
      fw.write(text);
      fw.close();
    }
    catch(IOException ioe) {
      System.out.println(ioe);
    }
  }    
  
    
  private void checkInput(String text) {
    boolean unknown = false;
    if (v.size() == 0) return;
    Vector v2 = (Vector)v.clone();
    v2.add(text);
    Collections.sort(v2);
    int pos = v2.indexOf(text);
    int next = pos + 1;
    if (pos == v.size()) pos--;
    li.setSelectedIndex(pos);
    ensureIndexIsVisible(pos);
    if (next == v2.size()) unknown = true;
    else {
      String nextName = (String)v2.elementAt(next);
      unknown = (nextName.indexOf(text) == -1);
    }
    if (unknown) {
      tf.setForeground(Color.red);
      cellRenderer.toRed();
      li.clearSelection();
    }
    else {
      tf.setForeground(Color.black);
      cellRenderer.toBlack();
    }
    repaint();
  }
  
  
  private void select(boolean next) {
    if (v.size() == 0) return;
    int i = li.getSelectedIndex();
    if (i == -1) {
      if (next) li.setSelectedIndex(0);
      else li.setSelectedIndex(v.size()-1);
    }
    else {
      if (next && i < v.size()-1) li.setSelectedIndex(i + 1);
      if (!next && i > 0) li.setSelectedIndex(i - 1);
    }
    ensureIndexIsVisible(li.getSelectedIndex());
    tf.setText((String)li.getSelectedValue());
    toBlack();
  }
  
     
  private void fireDeleteItem() {
    for (int i = 0; i < listeners.size(); i++) {
      ((IndexListener)listeners.elementAt(i)).deleteItem();
    }
  }
  
        
  private void fireNewInput(boolean isKnown, String item) {
    for (int i = 0; i < listeners.size(); i++) {
      ((IndexListener)listeners.elementAt(i)).newInput(isKnown, item);
    }
  }
  
  
  private void ensureIndexIsVisible(int pos) {
    if (secondLine) {
      int wantedPos = li.getFirstVisibleIndex() + 1;
      Rectangle r = li.getCellBounds(0, 0);
      JScrollBar sb = scrollPane.getVerticalScrollBar();
      sb.setValue(sb.getValue() + r.height * (pos-wantedPos));      
    }
    else li.ensureIndexIsVisible(pos);
  }

  
  public void keyPressed(KeyEvent e) {}
  public void keyTyped(KeyEvent e) {}        
  public void mousePressed(MouseEvent e) {}
  public void mouseClicked(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
}